----------------------------- MODULE bank_video -----------------------------
(***************************************************************************)
(* The following statement imports the standard operators of arithmetic    *)
(* such as + and =< (less than or equals).  It also defines the operator   *)
(* ..  so that i..j is the set of all integers k with i =< k =< j.         *)
(***************************************************************************)
EXTENDS Integers, TLC

(***************************************************************************)
(* The following statement declares N and ACCOUNT_ID to be unspecified     *)
(* constant values.                                                        *)
(***************************************************************************)
CONSTANTS
     N          \*size of type VALUE
    ,ACCOUNT_ID \* set of possible bank account identities
    
ASSUME
   /\ N >= 0
 
VALUE == -N .. N   \* money amounts to deposit and withdraw
MAX == 5*N         \* max in any account  
   
VARIABLES
     total \* total money in the bank
    ,b     \* bank accounts [ID -> VALUE]
    ,ID    \* set of actual IDs in bank

vars == <<total, b, ID>>

TypeOk == 
    /\ total >= 0
    /\ ID \subseteq ACCOUNT_ID
    /\ b \in [ID -> Int]

BalanceOk == 
   \A id \in ID: b[id] >= 0
   
Init == 
    /\ total = 0 
    /\ b = << >> \* the empty sequence is an empty function
    /\ ID = {}
 --------------------------------------------------------------------------   

\* Actions

add(id) ==
    /\ id \notin ID
    /\ b' = b @@ id :> 0 \* add tuple (id,0) to bank function b
    /\ ID' = ID \union {id}
    /\ UNCHANGED total
\*    /\ Print (<<"add", id>>, TRUE)



deposit(id,v) ==
    /\ id \in ID
    /\ v \in VALUE /\ v > 0
\*    /\ b[id] + v <= MAX  -- what happens if this line is commented out?
    /\ total' = total + v
    /\ b' = [b EXCEPT ![id] = @ + v]
    /\ UNCHANGED ID
    
withdraw(id,v) ==
    /\ id \in ID
    /\ v \in VALUE /\ v > 0
    /\ v <= b[id]
    /\ total' = total - v
    /\ b' = [b EXCEPT ![id] = @ - v]
    /\ UNCHANGED ID
    
\*transfer(id1,id2,v) == ????

Next == 
    \/ (\E id \in ACCOUNT_ID: add(id))
    \/ (\E id \in ACCOUNT_ID, v  \in VALUE: deposit(id,v))
    \/ (\E id \in ACCOUNT_ID, v  \in VALUE: withdraw(id,v))
=============================================================================
\* Modification History
\* Last modified Tue Aug 21 20:51:58 EDT 2018 by jonathan
\* Created Sun Aug 19 17:32:20 EDT 2018 by jonathan
